
## LearnBayes, supporting Jim Albert's 'Bayesian Computation with R'

## helper function
tstatistic <- function(x,y) {
    m <- length(x)
    n <- length(y)
    sp <- sqrt(((m-1)*sd(x)^2 + (n-1)*sd(y)^2) / (m+n-2))
    t.stat <- (mean(x) - mean(y)) / (sp*sqrt(1/m + 1/n))
    return(t.stat)
}


## Section 1.3.3
## simulation algorithm for normal populations
sim1_3_3_R <- function() {
    alpha <- .1; m <- 10; n <- 10 # sets alpha, m, n
    N <- 10000                    # sets nb of sims
    n.reject <- 0                 # number of rejections
    crit <- qt(1-alpha/2,n+m-2)
    for (i in 1:N) {
        x <- rnorm(m,mean=0,sd=1) # simulates xs from population 1
        y <- rnorm(n,mean=0,sd=1) # simulates ys from population 2
        t.stat <- tstatistic(x,y) # computes the t statistic
        if (abs(t.stat)>crit)
            n.reject=n.reject+1   # reject if |t| exceeds critical pt
    }
    true.sig.level <- n.reject/N  # est. is proportion of rejections
}

require(compiler)
sim1_3_3_Rcomp <- cmpfun(sim1_3_3_R)

#s=readline(prompt="Type  <Return>   to continue : ")

# simulation algorithm for normal and exponential populations
# storing the values of the t statistic in vector tstat

## m <- 10; n <- 10
## my.tsimulation <- function()
##     tstatistic(rnorm(m,mean=10,sd=2), rexp(n,rate=1/10))
## tstat.vector <- replicate(10000, my.tsimulation())

## pdf("/tmp/demoAlbert.pdf")
## plot(density(tstat.vector),xlim=c(-5,8),ylim=c(0,.4),lwd=3)
## curve(dt(x,df=18),add=TRUE)
## dev.off()

## my.tsimulation()


## same in C++: using arma

## quick test
## cpp_tstatistic <- cxxfunction(signature(xs="numeric", ys="numeric"), plugin="RcppArmadillo", body='
##    arma::vec x = Rcpp::as<arma::vec>(xs);
##    arma::vec y = Rcpp::as<arma::vec>(ys);
##    int m = x.n_elem;
##    int n = y.n_elem;
##    double sp = sqrt( ( (m-1.0)*pow(stddev(x),2) + (n-1)*pow(stddev(y),2) ) / (m+n-2.0) );
##    double t_stat = (mean(x) - mean(y))/(sp*sqrt(1.0/m + 1.0/n));
##    return(Rcpp::wrap(t_stat));')

tstat_arma <- '
  double tstatistic(const arma::vec &x, const arma::vec &y) {
    int m = x.n_elem;
    int n = y.n_elem;
    double sp = sqrt( ( (m-1.0)*pow(stddev(x),2) +
                     (n-1)*pow(stddev(y),2) ) / (m+n-2.0) );
    double t_stat = (mean(x)-mean(y))/(sp*sqrt(1.0/m+1.0/n));
    return(t_stat);
  }
'

require(inline)
sim1_3_3_arma <- cxxfunction(, plugin="RcppArmadillo",
                             inc=tstat_arma, body='
  RNGScope scope;     // properly deal with RNGs
  double alpha = 0.1;
  int m = 10, n = 10; // sets alpha, m, n
  int N = 10000;      // sets the number of sims
  double n_reject = 0; // counter of num. of rejects
  double crit = ::Rf_qt(1.0-alpha/2.0, n+m-2.0,true,false);
  for (int i=0; i<N; i++)  {
    NumericVector x = rnorm(m, 0, 1);   // sim xs from pop 1
    NumericVector y = rnorm(n, 0, 1);   // sim ys from pop 2
    double t_stat = tstatistic(Rcpp::as<arma::vec>(x),
                               Rcpp::as<arma::vec>(y));
    if (fabs(t_stat) > crit)
      n_reject++;           // reject if |t| exceeds critical pt
  }
  double true_sig_level = 1.0*n_reject / N; // est. prop rejects
  return(wrap(true_sig_level));
')


## now with Rcpp 0.9.4.1 (unreleased !!) which adds mean(), sd(), var()

tstat_rcpp <- '
 double tstatistic(const NumericVector &x,
                   const NumericVector &y) {
   int m = x.size();
   int n = y.size();
   double sp = sqrt( ( (m-1.0)*pow(sd(x),2) +
                    (n-1)*pow(sd(y),2) ) / (m+n-2.0) );
   double t_stat = (mean(x)-mean(y))/(sp*sqrt(1.0/m+1.0/n));
   return(t_stat);
 }
'

sim1_3_3_rcpp <- cxxfunction(, plugin="Rcpp",
                             inc=tstat_rcpp, body='
  RNGScope scope;     // properly deal with RNG settings
  double alpha = 0.1;
  int m = 10, n = 10; // sets alpha, m, n
  int N = 10000;      // sets the number of simulations
  double n_reject = 0; // counter of num. of rejections
  double crit = ::Rf_qt(1.0-alpha/2.0, n+m-2.0, true, false);
  for (int i=0; i<N; i++)  {
    NumericVector x = rnorm(m, 0, 1);   // sim xs from pop 1
    NumericVector y = rnorm(n, 0, 1);   // sim ys from pop 2
    double t_stat = tstatistic(x, y);
    if (fabs(t_stat) > crit)
      n_reject++;       // reject if |t| exceeds critical pt
  }
  double true_sig_level = 1.0*n_reject / N; // est. prop rejects
  return(wrap(true_sig_level));
')

library(rbenchmark)
res <- benchmark(sim1_3_3_R(),
                 sim1_3_3_Rcomp(),
                 sim1_3_3_arma(),
                 sim1_3_3_rcpp(),
                 columns=c("test", "replications",
                           "elapsed", "relative",
                           "user.self"),
                 order="relative")
res
